@@ -25,6 +25,7 @@ from __future__ import with_statement
 
 import ConfigParser
 import logging
+import itertools
 
 import gobject
 import pango
@@ -32,9 +33,13 @@ import gtk
 
 import gtk_toolbox
 import hildonize
+import gv_backend
 import null_backend
 
 
+_moduleLogger = logging.getLogger("gv_views")
+
+
 def make_ugly(prettynumber):
 	"""
 	function to take a phone number and strip out all non-numeric
@@ -802,7 +807,7 @@ class AccountInfo(object):
 			return
 
 		for number, description in callbackNumbers.iteritems():
-			self._callbackList.append(make_pretty(number))
+			self._callbackList.append((make_pretty(number), description))
 
 		if not self.get_selected_callback_number():
 			self._set_callback_number(self._defaultCallback)
@@ -812,7 +817,7 @@ class AccountInfo(object):
 			if not self._backend.is_valid_syntax(number) and 0 < len(number):
 				self._errorDisplay.push_message("%s is not a valid callback number" % number)
 			elif number == self._backend.get_callback_number():
-				logging.warning(
+				_moduleLogger.warning(
 					"Callback number already is %s" % (
 						self._backend.get_callback_number(),
 					),
@@ -823,7 +828,7 @@ class AccountInfo(object):
 					make_pretty(number), make_pretty(self._backend.get_callback_number())
 				)
 				self._callbackSelectButton.set_label(make_pretty(number))
-				logging.info(
+				_moduleLogger.info(
 					"Callback number set to %s" % (
 						self._backend.get_callback_number(),
 					),
@@ -845,16 +850,27 @@ class AccountInfo(object):
 		try:
 			actualSelection = make_pretty(self.get_selected_callback_number())
 
+			userOptions = dict(
+				(number, "%s (%s)" % (number, description))
+				for (number, description) in self._callbackList
+			)
+			defaultSelection = userOptions.get(actualSelection, actualSelection)
+
 			userSelection = hildonize.touch_selector_entry(
 				self._window,
 				"Callback Number",
-				self._callbackList,
-				actualSelection,
+				list(userOptions.itervalues()),
+				defaultSelection,
 			)
-			number = make_ugly(userSelection)
+			reversedUserOptions = dict(
+				itertools.izip(userOptions.itervalues(), userOptions.iterkeys())
+			)
+			selectedNumber = reversedUserOptions.get(userSelection, userSelection)
+
+			number = make_ugly(selectedNumber)
 			self._set_callback_number(number)
 		except RuntimeError, e:
-			logging.exception("%s" % str(e))
+			_moduleLogger.exception("%s" % str(e))
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -900,7 +916,7 @@ class AccountInfo(object):
 
 			self._update_alarm_settings(recurrence)
 		except RuntimeError, e:
-			logging.exception("%s" % str(e))
+			_moduleLogger.exception("%s" % str(e))
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -1062,6 +1078,11 @@ class RecentCallsView(object):
 				self._isPopulated = False
 				recentItems = []
 
+			recentItems = (
+				gv_backend.decorate_recent(data)
+				for data in gv_backend.sort_messages(recentItems)
+			)
+
 			for personName, phoneNumber, date, action in recentItems:
 				if not personName:
 					personName = "Unknown"
@@ -1195,6 +1216,8 @@ class MessagesView(object):
 		"""
 		pass
 
+	_MIN_MESSAGES_SHOWN = 4
+
 	def _idly_populate_messageview(self):
 		with gtk_toolbox.gtk_lock():
 			banner = hildonize.show_busy_banner_start(self._window, "Loading Messages")
@@ -1209,17 +1232,29 @@ class MessagesView(object):
 				self._isPopulated = False
 				messageItems = []
 
+			messageItems = (
+				gv_backend.decorate_message(message)
+				for message in gv_backend.sort_messages(messageItems)
+			)
+
 			for header, number, relativeDate, messages in messageItems:
 				prettyNumber = number[2:] if number.startswith("+1") else number
 				prettyNumber = make_pretty(prettyNumber)
 
 				firstMessage = "<b>%s - %s</b> <i>(%s)</i>" % (header, prettyNumber, relativeDate)
-				newMessages = [firstMessage]
-				newMessages.extend(messages)
+				expandedMessages = [firstMessage]
+				expandedMessages.extend(messages)
+				if (self._MIN_MESSAGES_SHOWN + 1) < len(messages):
+					firstMessage = "<b>%s - %s</b> <i>(%s)</i>" % (header, prettyNumber, relativeDate)
+					secondMessage = "<i>%d Messages Hidden...</i>" % (len(messages) - self._MIN_MESSAGES_SHOWN, )
+					collapsedMessages = [firstMessage, secondMessage]
+					collapsedMessages.extend(messages[-self._MIN_MESSAGES_SHOWN-1:-1])
+				else:
+					collapsedMessages = expandedMessages
 
 				number = make_ugly(number)
 
-				row = (number, relativeDate, header, "\n".join(newMessages), newMessages)
+				row = (number, relativeDate, header, "\n".join(collapsedMessages), expandedMessages)
 				with gtk_toolbox.gtk_lock():
 					self._messagemodel.append(row)
 		except Exception, e:
